<?php

if (!defined('ABSPATH')) {

	exit;
}

if (!class_exists('EDD_NovinPal_Gateway')) {

	/**
	 * Pay line Gateway for Easy Digital Downloads
	 *
	 * @author                NovinPal
	 * @subpackage            Gateways
	 */
	class EDD_NovinPal_Gateway
	{
		/**
		 * Gateway key name
		 *
		 * @var                string
		 */
		public $keyName;

		/**
		 * Initialize gateway and hook
		 *
		 * @return                void
		 */
		public function __construct()
		{
			$this->keyName = 'novinpal';

			add_filter('edd_payment_gateways', array($this, 'add'));
			add_action($this->format('edd_{key}_cc_form'), array($this, 'cc_form'));
			add_action($this->format('edd_gateway_{key}'), array($this, 'process'));
			add_action($this->format('edd_verify_{key}'), array($this, 'verify'));
			add_filter('edd_settings_gateways', array($this, 'settings'));

			add_action('edd_payment_receipt_after', array($this, 'receipt'));

			add_action('init', array($this, 'listen'));
		}

		/**
		 * Add gateway to list
		 *
		 * @param array $gateways Gateways array
		 * @return                array
		 */
		public function add($gateways)
		{
			global $edd_options;

			$gateways[$this->keyName] = array(
				'checkout_label' => isset($edd_options['novinpal_label']) ? $edd_options['novinpal_label'] : 'پرداخت آنلاین نوین پال',
				'admin_label'    => 'نوین پال'
			);

			return $gateways;
		}

		/**
		 * CC Form
		 * We don't need it anyway.
		 *
		 */
		public function cc_form()
		{
			return;
		}

		/**
		 * Process the payment
		 *
		 * @param array $purchase_data
		 * @return                void
		 */
		public function process($purchase_data)
		{
			global $edd_options;
			@ session_start();
			$payment = $this->insert_payment($purchase_data);

			if (!$payment) {

				edd_send_back_to_checkout('?payment-mode=' . $purchase_data['post_data']['edd-gateway']);
				return;
			}

			$callback = add_query_arg('verify_' . $this->keyName, '1', get_permalink($edd_options['success_page']));
			$description = 'پرداخت شماره #' . $payment . ' | ' . $purchase_data['user_info']['first_name'] . ' ' . $purchase_data['user_info']['last_name'];

			$amount = $this->novinpal_get_rial_amount(edd_get_currency(), intval($purchase_data['price']));

			$novinpal = new NovinpalEDD($this->getApiKey());
			$result = $novinpal->request(
				$amount,
				$callback,
				$payment,
				$description
			);

			if ($result['status'] == NovinpalEDD::NOVINPAL_STATUS_EMPTY_EDD) {

				edd_insert_payment_note($payment, 'cURL Error #:');
				edd_update_payment_status($payment, 'failed');
				edd_set_error('novinpal_connect_error', 'در اتصال به درگاه مشکلی پیش آمد');
				edd_send_back_to_checkout();
			}
			else if ($result['status'] == NovinpalEDD::NOVINPAL_STATUS_SUCCESS_EDD) {

				$refId = $result['data']['refId'];
				edd_insert_payment_note($payment, 'کد تراکنش نوین پال: ' . $refId);
				edd_update_payment_meta($payment, 'novinpal_refId', $refId);
				$_SESSION['np_payment'] = $payment;
				$novinpal->start($refId);
			}
			else {

				$message = ' تراکنش ناموفق بود- کد خطا: ' . (!empty($result['data']['errorCode']) ? $result['data']['errorCode'] : 'نا مشخص');
				edd_insert_payment_note($payment, $message);
				edd_insert_payment_note($payment, $this->error_reason($result['data']['errorCode']));
				edd_update_payment_status($payment, 'failed');
				edd_set_error('novinpal_connect_error', 'در اتصال به درگاه مشکلی پیش آمد');
				edd_send_back_to_checkout();
			}
		}

		/**
		 * Verify the payment
		 *
		 * @return                void
		 */
		public function verify()
		{

			global $edd_options;

			$invoiceNumber = isset($_GET['invoiceNumber']) ? sanitize_text_field($_GET['invoiceNumber']) : '';
			$status = !empty($_GET['success']) ? $_GET['success'] : 0;
			$returnTransactionId = !empty($_GET['refId']) ? $_GET['refId'] : 0;
			@ session_start();
			$payment = isset($_SESSION['np_payment']) ? edd_get_payment($_SESSION['np_payment']) : '';
			unset($_SESSION['np_payment']);

			if (empty($invoiceNumber) || !$payment || $payment->ID != $invoiceNumber) {

				wp_die('رکورد پرداخت موردنظر وجود ندارد!');
			}

			if ($payment->status == 'complete') {

				wp_die('تراکنش قبلاً تأیید شده است!');
			}

			if ($status != 1 || $_GET['code'] != 100) {

				edd_update_payment_status($payment->ID, 'failed');
				wp_redirect(get_permalink($edd_options['failure_page']));
				exit;
			}

			$amount = $this->novinpal_get_rial_amount(edd_get_currency(), edd_get_payment_amount($payment->ID));

			$novinpal = new NovinpalEDD($this->getApiKey());
			$verifyResponse = $novinpal->verify($returnTransactionId, $amount);

			if ($verifyResponse['status'] == NovinpalEDD::NOVINPAL_STATUS_VERIFIED_BEFORE_EDD) {

				$refId = $verifyResponse['data']['refId'];

				edd_empty_cart();

				edd_insert_payment_note($payment->ID, 'این تراکنش قبلا تایید شده است. شماره پیگیری: ' . $refId);
				edd_update_payment_meta($payment->ID, 'novinpal_refid', $refId);
				edd_update_payment_status($payment->ID, 'publish');
				edd_send_to_success_page();
			}
			else if ($verifyResponse['status'] == NovinpalEDD::NOVINPAL_STATUS_SUCCESS_EDD) {

				$refId = $verifyResponse['data']['refId'];

				edd_empty_cart();

				if (version_compare(EDD_VERSION, '2.1', '>=')) {

					edd_set_payment_transaction_id($payment->ID, $_GET['refId']);
				}

				edd_insert_payment_note($payment->ID, 'شماره تراکنش بانکی: ' . $refId);
				edd_update_payment_meta($payment->ID, 'novinpal_refid', $refId);
				edd_update_payment_status($payment->ID, 'publish');
				edd_send_to_success_page();
			}
			else {

				edd_update_payment_status($payment->ID, 'failed');
				wp_redirect(get_permalink($edd_options['failure_page']));
			}
		}

		/**
		 * Receipt field for payment
		 *
		 * @param object $payment
		 * @return                void
		 */
		public function receipt($payment)
		{
			$refid = edd_get_payment_meta($payment->ID, 'novinpal_refid');

			if ($refid) {

				echo '<tr class="novinpal-ref-id-row enp-field"><td><strong>شماره تراکنش بانکی:</strong></td><td>' . $refid . '</td></tr>';
			}
		}

		/**
		 * Gateway settings
		 *
		 * @param array $settings
		 * @return                array
		 */
		public function settings($settings)
		{
			return array_merge($settings, array(
				$this->keyName . '_header'   => array(
					'id'   => $this->keyName . '_header',
					'type' => 'header',
					'name' => '<strong>درگاه <a href="https://novinpal.ir" target="_blank">نوین پال</a></strong>'
				),
				$this->keyName . '_api_key' => array(
					'id'   => $this->keyName . '_api_key',
					'name' => 'API Key',
					'type' => 'text',
					'size' => 'regular'
				)
			));
		}

		/**
		 * Format a string, replaces {key} with $keyname
		 *
		 * @param string $string To format
		 * @return            string Formatted
		 */
		private function format($string)
		{
			return str_replace('{key}', $this->keyName, $string);
		}

		/**
		 * Inserts a payment into database
		 *
		 * @param array $purchase_data
		 * @return            int $payment_id
		 */
		private function insert_payment($purchase_data)
		{
			global $edd_options;

			$payment_data = array(
				'price'        => $purchase_data['price'],
				'date'         => $purchase_data['date'],
				'user_email'   => $purchase_data['user_email'],
				'purchase_key' => $purchase_data['purchase_key'],
				'currency'     => $edd_options['currency'],
				'downloads'    => $purchase_data['downloads'],
				'user_info'    => $purchase_data['user_info'],
				'cart_details' => $purchase_data['cart_details'],
				'status'       => 'pending'
			);

			// record the pending payment
			$payment = edd_insert_payment($payment_data);

			return $payment;
		}

		/**
		 * Listen to incoming queries
		 *
		 * @return            void
		 */
		public function listen()
		{
			if (isset($_GET['verify_' . $this->keyName]) && $_GET['verify_' . $this->keyName]) {

				do_action('edd_verify_' . $this->keyName);
			}
		}

		/**
		 * Error reason for NovinPal
		 *
		 * @param int $error_id
		 * @return            string
		 */

		public function error_reason($error_id)
		{

			switch ($error_id) {
				case '100':
					return 'تراکنش با موفقیت انجام شد';

				case '101':
					return 'آی پی سایت با آی پی سایت شما در نوین پال متفاوت است';

				case '102':
					return 'کسب و کار مجاز نیست';

				case '103':
					return 'آدرس بازگشت از خرید با آدرس سایت شما در نوین پال متفاوت است';

				case '105':
					return 'تراکنش یافت نشد';

				case '106':
					return 'شماره پیگیری اشتباه است';

				case '109':
					return 'پرداخت ناموفق بود';

				case '110':
					return 'مقدار مبلغ اشتباه است';

				case '111':
					return 'کلید API اشتباه است';

				case '112':
					return 'پذیرنده غیرفعال است';

				case '113':
					return 'شماره سفارش اشتباه است';

				case '114':
					return 'متد فراخوانی شده اشتباه است';

				case '115':
					return 'درگاه تأیید نشده است';

				case '116':
					return 'درگاه غیرفعال است';

				case '117':
					return 'وضعیت درگاه رد شده است';

				case '118':
					return 'درگاه در وضعیت تعلیق شده است';

				case '119':
					return 'درگاه تعریف نشده است';

				case '120':
					return 'وضعیت صاحب درگاه تعلیق شده است';

				case '121':
					return 'اطلاعات صاحب درگاه تأیید نشده است';

				default:
					return 'خطای ناشناخته، با پشتیبانی نوین پال تماس بگیرید';
			}
		}

		public function getApiKey()
		{
			global $edd_options;

			if (isset($edd_options[$this->keyName . '_api_key'])) {

				return $edd_options[$this->keyName . '_api_key'];
			}

			return '';
		}

		protected function getToken($params)
		{

			return $this->sendRequestToNovinPal($this->getTokenUrl, $params);
		}

		protected function verifyTransaction($params)
		{

			return $this->sendRequestToNovinPal($this->verifyUrl, $params);
		}

		public function sendRequestToNovinPal($url, $params)
		{
			try {

				$number_of_connection_tries = 5;
				$response = null;
				while ($number_of_connection_tries > 0) {
					$response = wp_safe_remote_post($url, array(
						'body'    => $params,
						'headers' => [
							'User-Agent' => 'NovinPal Rest Api v1'
						]
					));

					if (!is_wp_error($response)) {

						break;
					}

					$number_of_connection_tries--;
				}

				$bodyJson = wp_remote_retrieve_body($response);

				$bodyArray = json_decode($bodyJson, true);

				if (!is_array($bodyArray))
					return false;

				return $bodyArray;

			} catch (Exception $ex) {

				return false;
			}
		}

		public function novinpal_get_rial_amount($currency, $amount)
		{

			switch (strtoupper($currency)) {
				case 'IRT':
					return $amount * 10;
				case 'IRHR':
					return $amount * 1000;
				case 'IRHT':
					return $amount * 10000;
				default:
					return $amount;
			}
		}
	}

}

new EDD_NovinPal_Gateway;
